import json
from aliyunsdkcore.client import AcsClient
from aliyunsdkcore.acs_exception.exceptions import ClientException
from aliyunsdkcore.acs_exception.exceptions import ServerException
from aliyunsdkvod.request.v20170321.GetPlayInfoRequest import GetPlayInfoRequest
from flask import request, jsonify, session, current_app
from sqlalchemy import func, desc, or_
from apps.models import *
from apps.view_attract import api_attract
from apps.utils.response_code import RET
from apps import db, constants, redis_store


# 获取左上角经济指标数据
def get_jjzb(district):
    '''经济指标数据'''
    years = [2021]  # 指定数据的年份，与数据库中数据对齐

    try:
        if not district or district == '晋城市':
            datas = City.query.filter_by(area='晋城市').filter(City.year.in_(years)).all()
        else:
            datas = City.query.filter_by(area=district).filter(City.year.in_(years)).all()

        df = [{"list": [{"nums": '%.2f亿元' % (data.GDP / 10000) if data.GDP else "-亿元", "name": "地区生产总值"},
                        {"nums": '%.2f' % data.addscale + "%" if data.addscale else "-%", "name": "规上工业增加值增速"},
                        {"nums": '%.2f亿元' % (data.investment / 10000) if data.investment else "-亿元",
                         "name": "固定资产投资"},
                        {"nums": '%.2f亿元' % (data.retail / 10000) if data.retail else "-亿元", "name": "社会消费品零售额"},
                        {"nums": '%.2f亿元' % (data.in_out / 10000) if data.in_out else "-亿元", "name": "进出口总额"},
                        {"nums": '%.2f亿元' % (data.public / 10000) if data.public else "-亿元", "name": "一般公共预算支出"},
                        {"nums": '%.2f元' % data.people_out if data.people_out else "-元", "name": "居民人均可支配收入"},
                        {"nums": '%.2f' % data.people_per if data.people_per else "-", "name": "居民消费价格指数"}],
               "desc": data.question if data.question else ""} for data in datas]
        # return jsonify(code=RET.OK, msg="查找成功", data=df)
        return df
    except Exception as e:
        current_app.logger.error(e)
        return jsonify(code=RET.DBERR, msg="数据库错误")

@api_attract.route("/index", methods=["GET"])
def menu():
    return 'hello! 你好！'

# 左上角企业数量各资质统计、左上角获取经济指标数据、产业机构分布饼状图数据统计
@api_attract.route("/menu", methods=["POST"])
def menu():
    '''左上角企业数量各资质统计、左上角获取经济指标数据、产业机构分布饼状图数据统计'''
    req_dict = request.get_json()
    district = req_dict.get("district")  # 区县或者开发区名称

    # 区县名称转换字典
    district_dict = {
                     '沁水市': '沁水市',
                     '高平市': '高平市',
                     '陵川市': '陵川市',
                     '阳城市': '阳城市',
                     '泽州市': '泽州市',
                     '城区': '城区'
                     }

    if not district or district == '晋城市':
        try:
            company = Company.query.filter_by(city="晋城市")
        except Exception as e:
            current_app.logger.error(e)
            return jsonify(code=RET.DBERR, msg="数据库错误")
    else:
        try:
            company = Company.query.filter(Company.induzone == district_dict[district], Company.city == "晋城市")
        except Exception as e:
            current_app.logger.error(e)
            return jsonify(code=RET.DBERR, msg="数据库错误")

    # 1. 左上角企业数量各资质统计
    try:
        # 晋城市
        t_c5 = company.filter_by(isfive="1").count()  # 500强
        ssqy = company.filter_by(quoted_company="1").count()  # 上市
        if district != "晋城市":
            gmys = Company.query.filter(Company.induzone == district_dict[district]).filter_by(
                scale="1").count()  # 规模(2-5) 622
        else:
            gmys = Company.query.filter_by(scale="1").count()
        zjtx = company.filter_by(zjtg="1").count()  # 专精特新
        gxjs = company.filter_by(high_new="1").count()  # 高新
        kjzx = company.filter_by(tbe="1").count()  # 科技型中小企业
        dengl = company.filter_by(dengl="1").count()  # 瞪羚
        dujs = company.filter_by(unicorn="1").count()  # 独角兽

        table_data = [
            {"name": "500强企业", "nums": t_c5},
            {"name": "上市企业", "nums": ssqy},
            {"name": "规模以上企业", "nums": gmys},
            {"name": "专精特新企业", "nums": zjtx},
            {"name": "高新技术企业", "nums": gxjs},
            {"name": "科技型中小企业", "nums": kjzx},
            {"name": "瞪羚企业", "nums": dengl},
            {"name": "独角兽企业", "nums": dujs},
        ]

        # 2. 左上角获取经济指标数据
        jjzb_data = get_jjzb(district)

        # 3. 产业机构分布饼状图数据统计
        count_all = company.count()
        if count_all == 0:
            count_all = 1
        count_mcq = company.filter_by(f_type=1).count()  # 煤层气
        count_gjd = company.filter_by(f_type=3).count()  # 光机电
        count_mtmhg = company.filter_by(f_type=4).count()  # 煤炭煤化工
        count_gtzz = company.filter_by(f_type=2).count()  # 钢铁铸造
        count_qyly = company.filter_by(f_type=5).count()  # 全域旅游和康养
        # count_xdfu = company.filter_by(f_type=6).count()  # 现代服务业
        # count_qt = count_all - count_mcq - count_gjd - count_mtmhg - count_gtzz - count_qyly - count_xdfu
        count_qt = count_all - count_mcq - count_gjd - count_mtmhg - count_gtzz - count_qyly

        mcq = int(count_mcq / count_all * 100)
        gjd = int(count_gjd / count_all * 100)
        mtmhg = int(count_mtmhg / count_all * 100)
        gtzz = int(count_gtzz / count_all * 100)
        qyly = int(count_qyly / count_all * 100)
        # xdfu = int(count_xdfu / count_all * 100)
        # qt = 100 - mcq - gjd - mtmhg - gtzz - qyly - xdfu
        qt = 100 - mcq - gjd - mtmhg - gtzz - qyly
        # 产业结构分布
        industry = [
            {"per": mtmhg, "name": "煤炭煤化工", "count": count_mtmhg},
            {"per": mcq, "name": "煤层气", "count": count_mcq},  # {百分比，value}
            {"per": gjd, "name": "光机电", "count": count_gjd},
            {"per": gtzz, "name": "钢铁铸造", "count": count_gtzz},
            {"per": qyly, "name": "全域旅游和康养", "count": count_qyly},
            # {"per": qt + xdfu, "name": "其他", "count": count_qt + count_xdfu},
            {"per": qt, "name": "其他", "count": count_qt},
        ]

        data = {
            "jjzb_data":jjzb_data,  # 左上角经济指标数据
            "table_data": table_data,  # 左上角企业数据统计
            "industry": industry  # 产业结构分布饼状图数据
        }

        return jsonify(code=RET.OK, msg="查找成功", data=data)
    except Exception as e:
        current_app.logger.error(e)
        return jsonify(code=RET.DBERR, msg="数据库错误")


# 招商引资作战图、项目数量/金额分布图
@api_attract.route("/project", methods=["POST"])
def project():
    req_dict = request.get_json()
    district = req_dict.get("district")  # 区县或者开发区名称
    try:
        project_map = []
        industry_money = []
        industry_nums = []
        list1 = ["沁水市", "高平市", "陵川市", "阳城市", "泽州市", "城区"]

        # 晋城市，点击区县之前默认数据
        if not district or district == '晋城市':
            for i in list1:
                # 1.招商引资作战图数据
                ss = Examine.query.filter_by(district=i).first()
                project_map.append({"name": i,
                                    "nums1": "{}亿元".format(ss.sign_finnish),
                                    "add1": ss.sign_grade,
                                    "nums2": "{}亿元".format(ss.start_finish),
                                    "add2": ss.start_grade,
                                    "nums3": "{}亿元".format(ss.invest_finish),
                                    "add3": ss.invest_grade
                                    })
                # 产业数量/金额分布图数据
                project_list = NewProject.query.filter_by(district=i).all()
                for project in project_list:
                    industry_money.append({"name": project.type,
                                           # "number": i.number,
                                           "money": project.money})
                    industry_nums.append({"name": project.type,
                                          # "money": i.money,
                                          "number": project.number})

        # 点击区县后的联动数据
        else:
            # 1.招商引资作战图数据
            ss = Examine.query.filter_by(district=district).first()
            project_map.append({"name": district,
                                "nums1": "{}亿元".format(ss.sign_finnish),
                                "add1": ss.sign_grade,
                                "nums2": "{}亿元".format(ss.start_finish),
                                "add2": ss.start_grade,
                                "nums3": "{}亿元".format(ss.invest_finish),
                                "add3": ss.invest_grade
                                })
            # 产业数量/金额分布图数据
            project_list = NewProject.query.filter_by(district=district).all()
            for project in project_list:
                industry_money.append({"name": project.type,
                                       # "number": i.number,
                                       "money": project.money})
                industry_nums.append({"name": project.type,
                                      # "money": i.money,
                                      "number": project.number})
    except Exception as e:
        current_app.logger.error(e)
        return jsonify(code=RET.DBERR, msg="数据库错误")

    data = {
        "project_map": project_map,  # 招商引资作战图
        "industry_nums": industry_nums,  # 已签约项目数量产业分布图
        "industry_money": industry_money,  # 已签约项目投资金额产业分布图
    }
    return jsonify(code=RET.OK, msg="获取成功", data=data)


# 政策和重点项目 --- 重点项目需要改名称
@api_attract.route("/propol", methods=["POST"])
def propol():
    req_dict = request.get_json()
    district = req_dict.get("district")  # 县区
    district_dict = {'沁水市': '沁水市',
                     '高平市': '高平市',
                     '陵川市': '陵川市',
                     '阳城市': '阳城市',
                     '泽州市': '泽州市',
                     '城区': '城区'}
    try:
        project = Project.query.filter_by()  # 重点项目
        policy = InduPolicy.query.filter_by()  # 政策

        if not district and district == "晋城市":
            # name_query = "project-propol"
            # if redis_store.get(name_query) is not None:
            #     data = json.loads(redis_store.get(name_query))
            #     return jsonify(code=RET.OK, msg="获取成功", data=data)
            project = project.filter_by()
            policy_province = policy.filter_by(district="山西省").order_by(InduPolicy.pubdate.desc())
            policy = policy.filter_by(district="晋城市").order_by(InduPolicy.pubdate.desc())
            policy_2 = [{"name": i.name, "id": i.id, "distict": "山西省"} for i in policy_province.all()]
            data_policy = [{"name": i.name, "id": i.id, "distict": "太原市"} for i in policy.all()]
            data_project = [{"name": i.name, "id": i.id} for i in project.all()]
            data = {
                "project": data_project,
                "policy": data_policy,
                "policy_province": policy_2
            }
            # redis缓存
            # redis_store.set(name_query, json.dumps(data))
            # redis_store.expire(name_query, 1 * 24 * 3600)
        else:  # 区县、开发区
            project = project.filter_by(district=district_dict[district])
            policy = policy.filter_by(district=district_dict[district]).order_by(InduPolicy.pubdate.desc())
            data_policy = [{"name": i.name, "id": i.id, "distict": district} for i in policy.all()]
            data_project = [{"name": i.name, "id": i.id} for i in project.all()]
            data = {
                "project": data_project,
                "policy": data_policy,

            }

        return jsonify(code=RET.OK, msg="查询成功", data=data)
    except Exception as e:
        current_app.logger.error(e)
        return jsonify(code=RET.DATAERR, msg="数据库错误")


# 获取区县的详情
@api_attract.route("/getInfo", methods=["POST"])
def get_info():
    district_dict = {'晋城市': '晋城市',
                     '沁水市': '沁水市',
                     '高平市': '高平市',
                     '陵川市': '陵川市',
                     '阳城市': '阳城市',
                     '泽州市': '泽州市',
                     '城区': '城区'
                     }

    req_dic = request.get_json()
    district = req_dic.get("district")  # 地区

    if not district:
        return jsonify(code=RET.PARAMERR, msg="参数不完整")

    try:
        city = City.query.filter_by(area=district_dict[district]).first()
        info = city.info
        return jsonify(code=RET.OK, msg="查询成功", data=info)
    except Exception as e:
        current_app.logger.error(e)
        return jsonify(code=RET.DBERR, msg="数据库错误")


# 获取区县视频url
@api_attract.route("/video", methods=["POST"])
def attract_video():
    req_dic = request.get_json()
    district = req_dic.get("district")

    if not district:
        return jsonify(code=RET.PARAMERR, msg="参数不全")

    district_dict = {'晋城市': '晋城市',
                     '沁水市': '沁水市',
                     '高平市': '高平市',
                     '陵川市': '陵川市',
                     '阳城市': '阳城市',
                     '泽州市': '泽州市',
                     '城区': '城区'
                     }

    if district not in district_dict:
        return jsonify(code=RET.DATAERR, msg="区县名称有误，没有相关视频")

    district = district_dict[district]
    video = Video.query.filter_by(district=district).first()
    if video:
        url = video.video_id
        if url:
            return jsonify(code=RET.OK, msg="查询成功", data={"url": url})

    return jsonify(code=RET.NODATA, msg="暂无视频", data={"url": ""})


# 获取重点项目的详情，详情页
@api_attract.route("/keyProject", methods=["POST"])
def keyproject():
    req_dic = request.get_json()
    id = req_dic.get("id")
    if not id:
        return jsonify(code=RET.PARAMERR, msg="参数不完整")

    try:
        project = Project.query.get(id)
        data = {
            "name": project.name,
            "district": project.district,
            "type": project.type,
            "money": project.money,
            "background": project.background,
            "content": project.content,
            "way": project.way,
            "company": project.company,
            "contact": project.contact,
            "email": project.email
        }
        return jsonify(code=RET.OK, msg="查找成功", data=data)
    except Exception as e:
        current_app.logger.error(e)
        return jsonify(code=RET.DBERR, msg="数据库错误")


# 政策详情
@api_attract.route("/policyDetails", methods=["POST"])
def policy_details():
    req_dic = request.get_json()
    _id = req_dic.get("id")
    if not _id:
        return jsonify(code=RET.PARAMERR, msg="参数不全")
    try:
        policy = InduPolicy.query.get(_id)
        data = {"name": policy.name,
                "file": policy.file if policy.file else "-",
                "category": policy.category if policy.category else "-",
                "org": policy.org if policy.org else "-",
                "industry": policy.industry if policy.industry else "-",
                "district": policy.district if policy.district else "-",
                "pubdate": str(policy.pubdate)[0:10] if policy.pubdate else "-",
                "url": policy.url if policy.pubdate else "-",
                "post_num": policy.post_num if policy.post_num else ""}
        return jsonify(code=RET.OK, msg="查找成功", data=data)
    except Exception as e:
        current_app.logger.error(e)
        return jsonify(code=RET.DBERR, msg="数据库错误")


# 列表展示左上角企业
@api_attract.route("/showListb", methods=["POST"])
def showListb():
    district_dict = {'晋城市': '晋城市',
                     '沁水市': '沁水市',
                     '高平市': '高平市',
                     '陵川市': '陵川市',
                     '阳城市': '阳城市',
                     '泽州市': '泽州市',
                     '城区': '城区'
                     }

    req_dic = request.get_json()
    type = req_dic.get("type")  # 1234567 （优质企业-500强-上市-规模-专精特新-高新-科技中小）
    classify = req_dic.get("classify")  # 1234  企业-学校-科研机构-传新平台
    district = req_dic.get("district")

    page = req_dic.get("page")
    perpage = req_dic.get("perpage")

    if not all([type, classify, district]):
        return jsonify(code=RET.PARAMERR, msg="参数错误")

    if not page:
        page = 1
    if not perpage:
        perpage = 20
    try:
        # 企业
        if classify == 1:
            if district == "太原市":
                company = Company.query.filter_by(city="太原市")
            else:
                # company = Company.query.filter(or_(Company.district == district_dict[district],
                #                                    Company.induzone == district_dict[district]))
                company = Company.query.filter_by(city="太原市").filter(Company.induzone == district_dict[district])
            size = company.count()
            # if type == 1:
            #     company = company.order_by(Company.isfive.desc()).order_by(Company.tbe.desc()).order_by(
            #         Company.high_new.desc()).order_by(
            #         Company.quoted_company.desc()).order_by(Company.scale.desc())
            if type == 1:
                company = company.filter_by(isfive=1)
                size = company.count()
            if type == 2:
                company = company.filter_by(quoted_company=1)
                size = company.count()
            if type == 3:
                # company = company.filter_by(scale=1)
                if district == "晋城市":
                    company = Company.query.filter_by(scale="1")
                else:
                    company = Company.query.filter(Company.induzone == district_dict[district]).filter_by(scale="1")
                size = company.count()
            if type == 4:
                company = company.filter_by(zjtg=1)
                size = company.count()
            if type == 5:
                company = company.filter_by(high_new=1)
                size = company.count()
            if type == 6:
                company = company.filter_by(tbe=1)
                size = company.count()
            if type == 7:
                company = company.filter_by(dengl=1)
                size = company.count()
            if type == 8:
                company = company.filter_by(unicorn=1)
                size = company.count()
            company = company.paginate(page, perpage).items
            data = [{
                "id": i.id,
                "name": i.company_name,
                "industry": i.company_industry if i.company_industry else "-",
                "product": i.product if i.product else "-",
                "build_date": i.build_date.strftime("%Y-%m-%d") if i.build_date else "-",
                # "capital": i.capital if i.capital else "-",
                "isfive": "500强企业" if i.isfive == "1" else "",
                # "quoted_company": "上市企业" if i.quoted_company == "1" else "",
                "quoted_company": i.public_sector if i.public_sector != "空" else "",
                "scale": "规模以上企业" if i.scale == "1" else "",
                "zjtg": "专精特新企业" if i.zjtg == "1" else "",
                "high_new": "高新技术企业" if i.high_new == "1" else "",
                "tbe": "科技型中小企业" if i.tbe == "1" else "",
                "dengl": "瞪羚企业" if i.dengl == "1" else "",
                "unicorn": "独角兽企业" if i.unicorn == "1" else "",
            } for i in company]
            data = {"data_list": data, "size": size}
        # 学校
        # elif classify == 2:
        #     if district == "太原市":
        #         college = College.query.filter()
        #     else:
        #         college = College.query.filter(or_(College.admin == district_dict[district],
        #                                            College.developarea == district_dict[district]))
        #     if type == 1:
        #         college = college.filter(or_(College.cateid == 1, College.cateid == 2))
        #     if type == 2:
        #         college = college.filter(College.cateid == 3)
        #     size = college.count()
        #     college = college.paginate(page, perpage).items
        #     data = [{
        #         "id": i.id,
        #         "name": i.name,
        #         "cateone": 1,
        #         "category": i.category if i.category else "-",
        #         "nature": i.nature if i.nature else "-",
        #         "buildate": i.buildate if i.buildate else "-",
        #     } for i in college]
        #     data = {"data_list": data, "size": size}
        # # 科研机构:
        # elif classify == 3:
        #     if district == "太原市":
        #         scientific = Scientific.query.filter_by()
        #     else:
        #         scientific = Scientific.query.filter(or_(Scientific.admin == district_dict[district],
        #                                                  Scientific.developarea == district_dict[district]))
        #     size = scientific.count()
        #     scientific = scientific.paginate(page, perpage).items
        #     data = [{
        #         "id": i.id,
        #         "name": i.name,
        #         "cateone": 2,
        #         "type": "科研机构",
        #         "telephone": i.telephone if i.telephone else "-",
        #     } for i in scientific]
        #     data = {"data_list": data, "size": size}
        # # 创新平台:
        # elif classify == 4:
        #     if district == "太原市":
        #         lab = Lab.query.filter_by()
        #     else:
        #         lab = Lab.query.filter(or_(Lab.admin == district_dict[district],
        #                                    Lab.developarea == district_dict[district]))
        #     size = lab.count()
        #     lab = lab.paginate(page, perpage).items
        #     data = [{
        #         "id": i.id,
        #         "name": i.name,
        #         "type": i.cate,
        #         "cateone": 3
        #     } for i in lab]
        #     data = {"data_list": data, "size": size}
        else:
            data = {}
        return jsonify(code=RET.OK, msg="查找成功", data=data)
    except Exception as e:
        current_app.logger.error(e)
        return jsonify(code=RET.DBERR, msg="数据库错误")


# 企业信息详情(招商驾驶舱左上角各类企业详情)
@api_attract.route('/companyDetail', methods=['POST'])
def enterprise_detail():
    '''
    企业信息详情
    :return:
    '''

    req_dict = request.get_json()
    _id = req_dict.get("id")  # 企业id

    # 校验参数完整性
    if not all([_id]):
        return jsonify(code=RET.PARAMERR, msg="参数不完整")

    try:
        company = Company.query.get(_id)
        if company:
            data = {"id": company.id,
                    "company_name": company.company_name,
                    "telephone": company.telephone if company.telephone else "-",
                    "web_site": company.web_site if company.web_site else "-",
                    "email": company.email if company.email else "-",
                    "address": company.address if company.address else "-",
                    "jwd": {"lng": company.lng if company.lng else "-",
                            "lat": company.lat if company.lat else "-"},
                    "company_info": company.company_info if company.company_info else "-",
                    "high_new": "高新技术企业" if company.high_new else "-",
                    "legal": company.legal if company.legal else "-",
                    "status": company.status if company.status else "-",
                    "build_date": str(company.build_date)[:10] if company.build_date else "-",
                    "capital": company.capital if company.capital else "-",
                    "social_code": company.social_code if company.social_code else "-",
                    "taking": company.takingn if company.takingn else "-",
                    "bao": company.bao_num if company.bao_num else "-",
                    "entype": company.entype if company.entype else "-",
                    "industry": company.company_industry if company.company_industry else "-",
                    "scope": company.business_scope if company.business_scope else "-",
                    "collect": "1",
                    "choice": "3"
                    }
        else:
            return jsonify(code=RET.NODATA, msg="查无数据")
    except Exception as e:
        current_app.logger.error(e)
        return jsonify(code=RET.DBERR, msg="数据库查询错误")
    return jsonify(code=RET.OK, msg="获取成功", data=data)


# 删除缓存
@api_attract.route("/deleteX", methods=["GET"])
def deleteX():
    redis_store.delete("project-propol")
    return jsonify(code=RET.OK, msg="删除成功")


# 删除缓存
@api_attract.route("/vis", methods=["GET"])
def vis():
    redis_store.delete("project-propol")
    return jsonify(code=RET.OK, msg={"remote_addr": request.remote_addr,
                                     "X-Real-Ip": request.headers.get("X-Real-Ip"),
                                     "X-Forwarded-For": request.headers.get("X-Forwarded-For"),
                                     "HOST": request.headers.get("Host"),
                                     "X-Forwarded-Proto": request.headers.get("X-Forwarded-Proto"),
                                     })
